@@ -1,4 +1,4 @@
-from typing import Iterable
+from collections.abc import Iterable
 
 from redis.client import Pipeline
 from redis.exceptions import WatchError
