@@ -38,3 +38,15 @@ class TimeoutFormatError(Exception):
 
 class AbandonedJobError(Exception):
     pass
+
+
+class SchedulerNotFound(Exception):
+    pass
+
+
+class DuplicateSchedulerError(Exception):
+    pass
+
+
+class StopRequested(Exception):
+    pass
