@@ -8,37 +8,36 @@ terminal colorizing code, originally by Georg Brandl.
 import calendar
 import datetime
 import importlib
+import inspect
 import logging
 import numbers
+import os
 import warnings
+from collections.abc import Generator, Iterable, Sequence
 
 # TODO: Change import path to "collections.abc" after we stop supporting Python 3.8
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
-    Generator,
-    Iterable,
-    List,
     Optional,
-    Sequence,
-    Tuple,
-    Type,
     TypeVar,
     Union,
     overload,
 )
 
-if TYPE_CHECKING:
-    from redis import Redis
-
-    from .queue import Queue
-
 from redis.exceptions import ResponseError
 
 from .exceptions import TimeoutFormatError
 
+if TYPE_CHECKING:
+    from redis import Redis
+
+    from .job import Job
+    from .queue import Queue
+    from .worker import Worker
+
+
 _T = TypeVar('_T')
 _O = TypeVar('_O', bound=object)
 ObjOrStr = Union[_O, str]
@@ -47,7 +46,34 @@ ObjOrStr = Union[_O, str]
 logger = logging.getLogger(__name__)
 
 
-def compact(lst: Iterable[Optional[_T]]) -> List[_T]:
+def resolve_function_reference(func) -> tuple[Any, str]:
+    """Resolve a function reference into instance and function name components.
+
+    Args:
+        func: The function reference to resolve - can be a method, function,
+             builtin, string path, or callable class instance.
+
+    Returns:
+        A tuple of (instance, func_name) where:
+        - instance: The object instance (for methods/callable instances) or None
+        - func_name: The string representation of the function name/path
+
+    Raises:
+        TypeError: If func is not a valid callable or string reference.
+    """
+    if inspect.ismethod(func):
+        return func.__self__, func.__name__
+    elif inspect.isfunction(func) or inspect.isbuiltin(func):
+        return None, f'{func.__module__}.{func.__qualname__}'
+    elif isinstance(func, str):
+        return None, as_text(func)
+    elif not inspect.isclass(func) and hasattr(func, '__call__'):  # a callable class instance
+        return func, '__call__'
+    else:
+        raise TypeError(f'Expected a callable or a string, but got: {func}')
+
+
+def compact(lst: Iterable[Optional[_T]]) -> list[_T]:
     """Excludes `None` values from a list-like object.
 
     Args:
@@ -79,16 +105,20 @@ def as_text(v: Union[bytes, str]) -> str:
         raise ValueError('Unknown type %r' % type(v))
 
 
-def decode_redis_hash(h: Dict[Union[bytes, str], _T]) -> Dict[str, _T]:
+def decode_redis_hash(h: dict[Union[bytes, str], Any], *, decode_values: bool = False) -> dict[str, Any]:
     """Decodes the Redis hash, ensuring that keys are strings
     Most importantly, decodes bytes strings, ensuring the dict has str keys.
 
     Args:
         h (Dict[Any, Any]): The Redis hash
+        decode_values (bool): If True, also decode values to strings using as_text(). Defaults to False.
 
     Returns:
         Dict[str, Any]: The decoded Redis data (Dictionary)
+        When decode_values=True, returns Dict[str, str]
     """
+    if decode_values:
+        return dict((as_text(k), as_text(v)) for k, v in h.items())
     return dict((as_text(k), v) for k, v in h.items())
 
 
@@ -145,6 +175,121 @@ def import_attribute(name: str) -> Callable[..., Any]:
     return getattr(attribute_owner, attribute_name)
 
 
+def import_worker_class(name: str) -> type['Worker']:
+    """Import a worker class from a dotted path name."""
+    cls = import_attribute(name)
+
+    if not isinstance(cls, type):
+        raise ValueError(f'Invalid worker class: {name}')
+
+    from .worker import Worker
+
+    if not issubclass(cls, Worker):
+        raise ValueError(f'Invalid worker class: {name}')
+
+    return cls
+
+
+def import_job_class(name: str) -> type['Job']:
+    """Import a job class from a dotted path name."""
+    cls = import_attribute(name)
+
+    if not isinstance(cls, type):
+        raise ValueError(f'Invalid job class: {name}')
+
+    from .job import Job
+
+    if not issubclass(cls, Job):
+        raise ValueError(f'Invalid job class: {name}')
+
+    return cls
+
+
+def import_queue_class(name: str) -> type['Queue']:
+    """Import a queue class from a dotted path name."""
+    cls = import_attribute(name)
+
+    if not isinstance(cls, type):
+        raise ValueError(f'Invalid queue class: {name}')
+
+    from .queue import Queue
+
+    if not issubclass(cls, Queue):
+        raise ValueError(f'Invalid queue class: {name}')
+
+    return cls
+
+
+def normalize_config_path(config_path: str) -> str:
+    """Normalize configuration path to dotted module path format.
+
+    Converts file paths like 'directory/config_file.py' or 'directory.config_file'
+    to dotted module paths like 'directory.config_file' for use with importlib.import_module().
+
+    Args:
+        config_path: Either a file path (e.g., 'app/cron_config.py', 'app/cron_config')
+                    or a dotted module path (e.g., 'app.cron_config')
+
+    Returns:
+        A dotted module path suitable for importlib.import_module()
+
+    Examples:
+        normalize_config_path('app/cron_config.py') -> 'app.cron_config'
+        normalize_config_path('app/cron_config') -> 'app.cron_config'
+        normalize_config_path('app.cron_config') -> 'app.cron_config'
+        normalize_config_path('/abs/path/to/config.py') -> 'abs.path.to.config'
+    """
+    # Check if it's already a dotted path (no path separators and no .py extension)
+    is_file_path = os.path.sep in config_path or config_path.endswith('.py')
+
+    if not is_file_path:
+        # Already a dotted module path, return as-is
+        return config_path
+
+    # Convert file path to dotted module path
+    normalized = config_path
+
+    # Remove .py extension if present
+    if normalized.endswith('.py'):
+        normalized = normalized[:-3]
+
+    # Handle absolute paths by removing leading separator
+    if normalized.startswith(os.path.sep):
+        normalized = normalized[1:]
+
+    # Replace path separators with dots
+    normalized = normalized.replace(os.path.sep, '.')
+
+    return normalized
+
+
+def validate_absolute_path(file_path: str) -> str:
+    """Validate that an absolute file path exists and points to a file.
+
+    Args:
+        file_path: The absolute file path to validate
+
+    Returns:
+        The same file path if validation passes (for chaining)
+
+    Raises:
+        FileNotFoundError: If the file does not exist
+        IsADirectoryError: If the path points to a directory instead of a file
+
+    Examples:
+        validate_absolute_path('/path/to/config.py')  # Returns '/path/to/config.py'
+        validate_absolute_path('/path/to/missing.py')  # Raises FileNotFoundError
+        validate_absolute_path('/path/to/directory')   # Raises IsADirectoryError
+    """
+    if not os.path.exists(file_path):
+        raise FileNotFoundError(f"Configuration file not found at '{file_path}'")
+
+    if not os.path.isfile(file_path):
+        raise IsADirectoryError(f"Configuration path points to a directory, not a file: '{file_path}'")
+
+    return file_path
+
+
 def now() -> datetime.datetime:
     """Return now in UTC"""
     return datetime.datetime.now(datetime.timezone.utc)
@@ -179,9 +324,17 @@ def is_nonstring_iterable(obj: Any) -> bool:
 
 
 @overload
-def ensure_job_list(obj: ObjOrStr) -> List[ObjOrStr]: ...
+def ensure_job_list(obj: str) -> list[str]: ...
 @overload
-def ensure_job_list(obj: Iterable[ObjOrStr]) -> List[ObjOrStr]: ...
+def ensure_job_list(obj: 'Job') -> list['Job']: ...
+@overload
+def ensure_job_list(obj: Union['Job', str, Sequence[Union['Job', str]]]) -> list[Union['Job', str]]: ...
+@overload
+def ensure_job_list(obj: Iterable[_O]) -> list[_O]: ...
+@overload
+def ensure_job_list(obj: _O) -> list[_O]: ...
+
+
 def ensure_job_list(obj):
     """When passed an iterable of objects, convert to list, otherwise, it returns
     a list with just that object in it.
@@ -206,7 +359,7 @@ def current_timestamp() -> int:
     return calendar.timegm(now().utctimetuple())
 
 
-def backend_class(holder, default_name, override=None) -> Type:
+def backend_class(holder, default_name, override=None) -> type:
     """Get a backend class using its default attribute name or an override
 
     Args:
@@ -225,11 +378,14 @@ def backend_class(holder, default_name, override=None) -> Type:
         return override
 
 
-def str_to_date(date_str: Optional[bytes]) -> Optional[datetime.datetime]:
+def str_to_date(date_str: Union[bytes, str]) -> datetime.datetime:
     if not date_str:
-        return None
+        raise ValueError('Empty string or bytestring provided')
     else:
-        return utcparse(date_str.decode())
+        if isinstance(date_str, bytes):
+            return utcparse(date_str.decode())
+        else:
+            return utcparse(date_str)
 
 
 def parse_timeout(timeout: Optional[Union[int, float, str]]) -> Optional[int]:
@@ -237,6 +393,7 @@ def parse_timeout(timeout: Optional[Union[int, float, str]]) -> Optional[int]:
     if not isinstance(timeout, numbers.Integral) and timeout is not None:
         try:
             timeout = int(timeout)
+            return timeout
         except ValueError:
             assert isinstance(timeout, str)
             digit, unit = timeout[:-1], (timeout[-1:]).lower()
@@ -250,10 +407,10 @@ def parse_timeout(timeout: Optional[Union[int, float, str]]) -> Optional[int]:
                     'such as "1h", "23m".'
                 )
 
-    return timeout
+    return int(timeout) if timeout is not None else None
 
 
-def get_version(connection: 'Redis') -> Tuple[int, int, int]:
+def get_version(connection: 'Redis') -> tuple[int, int, int]:
     """
     Returns tuple of Redis server version.
     This function also correctly handles 4 digit redis server versions.
@@ -326,7 +483,7 @@ def truncate_long_string(data: str, max_length: Optional[int] = None) -> str:
 
 
 def get_call_string(
-    func_name: Optional[str], args: Any, kwargs: Dict[Any, Any], max_length: Optional[int] = None
+    func_name: Optional[str], args: Any, kwargs: dict[Any, Any], max_length: Optional[int] = None
 ) -> Optional[str]:
     """
     Returns a string representation of the call, formatted as a regular
@@ -347,14 +504,14 @@ def get_call_string(
 
     arg_list = [as_text(truncate_long_string(repr(arg), max_length)) for arg in args]
 
-    list_kwargs = ['{0}={1}'.format(k, as_text(truncate_long_string(repr(v), max_length))) for k, v in kwargs.items()]
+    list_kwargs = [f'{k}={as_text(truncate_long_string(repr(v), max_length))}' for k, v in kwargs.items()]
     arg_list += sorted(list_kwargs)
     args = ', '.join(arg_list)
 
-    return '{0}({1})'.format(func_name, args)
+    return f'{func_name}({args})'
 
 
-def parse_names(queues_or_names: Iterable[Union[str, 'Queue']]) -> List[str]:
+def parse_names(queues_or_names: Iterable[Union[str, 'Queue']]) -> list[str]:
     """Given a iterable  of strings or queues, returns queue names"""
     from .queue import Queue
 
@@ -377,7 +534,7 @@ def get_connection_from_queues(queues_or_names: Iterable[Union[str, 'Queue']]) -
     return None
 
 
-def parse_composite_key(composite_key: str) -> Tuple[str, str]:
+def parse_composite_key(composite_key: str) -> tuple[str, str]:
     """Method returns a parsed composite key.
 
     Args:
